@@ -1,5 +1,5 @@
 /*
- * $Id: _psutil_mswindows.c 376 2009-05-06 11:30:47Z g.rodola $
+ * $Id: _psutil_mswindows.c 527 2010-03-01 01:12:58Z jloden $
  *
  * Windows platform-specific module methods for _psutil_mswindows
  */
@@ -8,12 +8,16 @@
 #include <windows.h>
 #include <Psapi.h>
 #include <time.h>
+#include <lm.h>
+#include <tlhelp32.h>
 
 #include "_psutil_mswindows.h"
 #include "arch/mswindows/security.h"
 #include "arch/mswindows/process_info.h"
 
 
+static PyObject *NoSuchProcessException;
+
 static PyMethodDef PsutilMethods[] =
 {
      {"get_pid_list", get_pid_list, METH_VARARGS,
@@ -46,21 +50,87 @@ static PyMethodDef PsutilMethods[] =
          "Return the amount of available virtual memory, in bytes"},
      {"get_system_cpu_times", get_system_cpu_times, METH_VARARGS,
          "Return system cpu times as a tuple (user, system, idle)"},
+    {"get_process_cwd", get_process_cwd, METH_VARARGS,
+        "Return process current working directory"},
+    {"suspend_process", suspend_process, METH_VARARGS,
+        "Suspend a process"},
+    {"resume_process", resume_process, METH_VARARGS,
+        "Resume a process"},
      {NULL, NULL, 0, NULL}
 };
 
-static PyObject *NoSuchProcessException;
+struct module_state {
+    PyObject *error;
+};
 
-PyMODINIT_FUNC
+#if PY_MAJOR_VERSION >= 3
+#define GETSTATE(m) ((struct module_state*)PyModule_GetState(m))
+#else
+#define GETSTATE(m) (&_state)
+static struct module_state _state;
+#endif
+
+#if PY_MAJOR_VERSION >= 3
+
+static int psutil_mswindows_traverse(PyObject *m, visitproc visit, void *arg) {
+    Py_VISIT(GETSTATE(m)->error);
+    return 0;
+}
+
+static int psutil_mswindows_clear(PyObject *m) {
+    Py_CLEAR(GETSTATE(m)->error);
+    return 0;
+}
+
+
+static struct PyModuleDef moduledef = {
+        PyModuleDef_HEAD_INIT,
+        "psutil_mswindows",
+        NULL,
+        sizeof(struct module_state),
+        PsutilMethods,
+        NULL,
+        psutil_mswindows_traverse,
+        psutil_mswindows_clear,
+        NULL
+};
+
+#define INITERROR return NULL
+
+PyObject *
+PyInit__psutil_mswindows(void)
+
+#else
+#define INITERROR return
+
+void
 init_psutil_mswindows(void)
+#endif
 {
-     PyObject *m;
-     m = Py_InitModule("_psutil_mswindows", PsutilMethods);
-     NoSuchProcessException = PyErr_NewException("_psutil_mswindows.NoSuchProcess",
+#if PY_MAJOR_VERSION >= 3
+    PyObject *module = PyModule_Create(&moduledef);
+#else
+    PyObject *module = Py_InitModule("_psutil_mswindows", PsutilMethods);
+#endif
+    if (module == NULL)
+        INITERROR;
+    struct module_state *st = GETSTATE(module);
+
+    st->error = PyErr_NewException("_psutil_mswindow.Error", NULL, NULL);
+    if (st->error == NULL) {
+        Py_DECREF(module);
+        INITERROR;
+    }
+
+    NoSuchProcessException = PyErr_NewException("_psutil_mswindows.NoSuchProcess",
                                                  NULL, NULL);
-     Py_INCREF(NoSuchProcessException);
-     PyModule_AddObject(m, "NoSuchProcess", NoSuchProcessException);
-     SetSeDebug();
+    Py_INCREF(NoSuchProcessException);
+    PyModule_AddObject(module, "NoSuchProcess", NoSuchProcessException);
+    SetSeDebug();
+
+#if PY_MAJOR_VERSION >= 3
+    return module;
+#endif
 }
 
 
@@ -162,7 +232,7 @@ static PyObject* kill_process(PyObject* self, PyObject* args)
     long pid;
     int pid_return;
     PyObject* ret;
-    ret = PyInt_FromLong(0);
+    ret = PyLong_FromLong(0);
 
     if (! PyArg_ParseTuple(args, "l", &pid)) {
         PyErr_SetString(PyExc_RuntimeError, "Invalid argument");
@@ -198,7 +268,7 @@ static PyObject* kill_process(PyObject* self, PyObject* args)
     }
 
     CloseHandle(hProcess);
-    return PyInt_FromLong(1);
+    return PyLong_FromLong(1);
 }
 
 
@@ -210,6 +280,7 @@ static PyObject* get_process_cpu_times(PyObject* self, PyObject* args)
     long        pid;
     HANDLE      hProcess;
     FILETIME    ftCreate, ftExit, ftKernel, ftUser;
+    DWORD       ProcessExitCode = 0;
 
     if (! PyArg_ParseTuple(args, "l", &pid)) {
         PyErr_SetString(PyExc_RuntimeError, "Invalid argument");
@@ -232,6 +303,12 @@ static PyObject* get_process_cpu_times(PyObject* self, PyObject* args)
         return NULL;
     }
 
+    /* make sure the process is running */
+    GetExitCodeProcess(hProcess, &ProcessExitCode);
+    if (ProcessExitCode == 0) {
+        return PyErr_Format(NoSuchProcessException, "No Process found with pid %lu", pid);
+    }
+
     if (! GetProcessTimes(hProcess, &ftCreate, &ftExit, &ftKernel, &ftUser)) {
         PyErr_SetFromWindowsErr(0);
         if (GetLastError() == ERROR_ACCESS_DENIED) {
@@ -277,6 +354,7 @@ static PyObject* get_process_create_time(PyObject* self, PyObject* args)
     long long unix_time;
     HANDLE      hProcess;
     FILETIME    ftCreate, ftExit, ftKernel, ftUser;
+    DWORD     ProcessExitCode = 0;
 
     if (! PyArg_ParseTuple(args, "l", &pid)) {
         PyErr_SetString(PyExc_RuntimeError, "Invalid argument");
@@ -299,6 +377,12 @@ static PyObject* get_process_create_time(PyObject* self, PyObject* args)
         return NULL;
     }
 
+    /* make sure the process is running */
+    GetExitCodeProcess(hProcess, &ProcessExitCode);
+    if (ProcessExitCode == 0) {
+        return PyErr_Format(NoSuchProcessException, "No Process found with pid %lu", pid);
+    }
+
     if (! GetProcessTimes(hProcess, &ftCreate, &ftExit, &ftKernel, &ftUser)) {
         PyErr_SetFromWindowsErr(0);
         if (GetLastError() == ERROR_ACCESS_DENIED) {
@@ -434,6 +518,7 @@ static PyObject* get_memory_info(PyObject* self, PyObject* args)
     HANDLE hProcess;
     PROCESS_MEMORY_COUNTERS counters;
     DWORD pid;
+    DWORD ProcessExitCode = 0;
 
 	if (! PyArg_ParseTuple(args, "l", &pid)) {
         return PyErr_Format(PyExc_RuntimeError,
@@ -445,6 +530,12 @@ static PyObject* get_memory_info(PyObject* self, PyObject* args)
         return PyErr_SetFromWindowsErr(0);
     }
 
+    /* make sure the process is running */
+    GetExitCodeProcess(hProcess, &ProcessExitCode);
+    if (ProcessExitCode == 0) {
+        return PyErr_Format(NoSuchProcessException, "No Process found with pid %lu", pid);
+    }
+
     if (! GetProcessMemoryInfo(hProcess, &counters, sizeof(counters)) ) {
         return PyErr_SetFromWindowsErr(0);
     }
@@ -600,7 +691,7 @@ static PyObject* get_system_cpu_times(PyObject* self, PyObject* args)
 		typedef DWORD (_stdcall *NTQSI_PROC) (int, PVOID, ULONG, PULONG);
 		NTQSI_PROC NtQuerySystemInformation;
 		HINSTANCE hNtDll;
-		SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION *sppi;
+		SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION *sppi = NULL;
 		SYSTEM_INFO si;
 		UINT i;
 
@@ -646,7 +737,7 @@ static PyObject* get_system_cpu_times(PyObject* self, PyObject* args)
                         // kernel time includes idle time on windows
                         // we return only busy kernel time subtracting idle
                         // time from kernel time
-						return Py_BuildValue("(ddd)", user,
+						return Py_BuildValue("(fff)", user,
                                                       kernel - idle,
                                                       idle
                                              );
@@ -668,3 +759,271 @@ static PyObject* get_system_cpu_times(PyObject* self, PyObject* args)
 
 	} // END GetSystemTimes NOT supported
 }
+
+
+
+
+/*
+ * Sid to User convertion
+ */
+BOOL SidToUser(PSID pSid, LPTSTR szUser, DWORD dwUserLen, LPTSTR szDomain, DWORD
+dwDomainLen, DWORD pid)
+{
+    SID_NAME_USE snuSIDNameUse;
+    DWORD dwULen;
+    DWORD dwDLen;
+
+    dwULen = dwUserLen;
+    dwDLen = dwDomainLen;
+
+    if ( IsValidSid( pSid ) )
+        {
+            // Get user and domain name based on SID
+            if ( LookupAccountSid( NULL, pSid, szUser, &dwULen, szDomain, &dwDLen, &snuSIDNameUse) )
+                {
+                    // LocalSystem processes are incorrectly reported as owned
+                    // by BUILTIN\Administrators We modify that behavior to
+                    // conform to standard taskmanager only if the process is
+                    // actually a System process
+                    if (is_system_proc(pid) == 1) {
+                        // default to *not* changing the data if we fail to
+                        // check for local system privileges, so only look for
+                        // definite confirmed system processes and ignore errors
+                        if ( lstrcmpi(szDomain, TEXT("builtin")) == 0 && lstrcmpi(szUser, TEXT("administrators")) == 0)
+                            {
+                                strncpy  (szUser, "SYSTEM", dwUserLen);
+                                strncpy  (szDomain, "NT AUTHORITY", dwDomainLen);
+                            }
+                    }
+                    return TRUE;
+                }
+        }
+
+    return FALSE;
+}
+
+typedef struct _UNICODE_STRING {
+    USHORT Length;
+    USHORT MaximumLength;
+    PWSTR Buffer;
+} UNICODE_STRING, *PUNICODE_STRING;
+
+/*
+ * Return process current working directory as a Python string.
+ */
+static PyObject* get_process_cwd(PyObject* self, PyObject* args)
+{
+    long pid;
+    HANDLE processHandle;
+    PVOID pebAddress;
+    PVOID rtlUserProcParamsAddress;
+    UNICODE_STRING currentDirectory;
+    WCHAR *currentDirectoryContent;
+    DWORD ProcessExitCode = 0;
+    PyObject *returnPyObj = NULL;
+    PyObject *cwd_from_wchar = NULL;
+    PyObject *cwd = NULL;
+
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        PyErr_SetString(PyExc_RuntimeError, "Invalid argument");
+        return NULL;
+    }
+
+    processHandle = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ,
+                                FALSE, pid);
+
+    /* check the process is running */
+    GetExitCodeProcess(processHandle, &ProcessExitCode);
+    if (ProcessExitCode == 0) {
+        return PyErr_Format(NoSuchProcessException, "No Process found with pid %lu", pid);
+    }
+
+    if (processHandle == 0)
+    {
+        PyErr_SetFromWindowsErr(0);
+        if (GetLastError() == ERROR_INVALID_PARAMETER) {
+            PyErr_Format(NoSuchProcessException,
+                         "No process found with pid %d", pid);
+        }
+        return NULL;
+    }
+
+    pebAddress = GetPebAddress(processHandle);
+
+    /* get the address of ProcessParameters */
+    if (!ReadProcessMemory(processHandle, (PCHAR)pebAddress + 0x10,
+        &rtlUserProcParamsAddress, sizeof(PVOID), NULL))
+    {
+        CloseHandle(processHandle);
+        if (GetLastError() == ERROR_PARTIAL_COPY) {
+            /* Usually means the process has gone in the meantime */
+            PyErr_Format(NoSuchProcessException, "No Process found with pid %lu", pid);
+            return NULL;
+        }
+        else {
+            return PyErr_SetFromWindowsErr(0);
+        }
+
+    }
+
+    /* read the currentDirectory UNICODE_STRING structure.
+       0x24 refers to "CurrentDirectoryPath" of RTL_USER_PROCESS_PARAMETERS
+       structure (http://wj32.wordpress.com/2009/01/24/howto-get-the-command-line-of-processes/)
+     */
+    if (!ReadProcessMemory(processHandle, (PCHAR)rtlUserProcParamsAddress + 0x24,
+        &currentDirectory, sizeof(currentDirectory), NULL))
+    {
+        CloseHandle(processHandle);
+        if (GetLastError() == ERROR_PARTIAL_COPY) {
+            /* Usually means the process has gone in the meantime */
+            PyErr_Format(NoSuchProcessException, "No Process found with pid %lu", pid);
+            return NULL;
+        }
+        else {
+            return PyErr_SetFromWindowsErr(0);
+        }
+    }
+
+    /* allocate memory to hold the command line */
+    currentDirectoryContent = (WCHAR *)malloc(currentDirectory.Length+1);
+
+    /* read the command line */
+    if (!ReadProcessMemory(processHandle, currentDirectory.Buffer,
+        currentDirectoryContent, currentDirectory.Length, NULL))
+    {
+        CloseHandle(processHandle);
+        free(currentDirectoryContent);
+
+        if (GetLastError() == ERROR_PARTIAL_COPY) {
+            /* Usually means the process has gone in the meantime */
+            PyErr_Format(NoSuchProcessException, "No Process found with pid %lu", pid);
+            return NULL;
+        }
+        else {
+            return PyErr_SetFromWindowsErr(0);
+        }
+    }
+
+    // null-terminate the string to prevent wcslen from returning incorrect length
+    // the length specifier is in characters, but commandLine.Length is in bytes
+    currentDirectoryContent[(currentDirectory.Length/sizeof(WCHAR))] = '\0';
+
+    // convert wchar array to a Python unicode string, and then to UTF8
+    cwd_from_wchar = PyUnicode_FromWideChar(currentDirectoryContent,
+                                            wcslen(currentDirectoryContent));
+
+    #if PY_MAJOR_VERSION >= 3
+        cwd = PyUnicode_FromObject(cwd_from_wchar);
+    #else
+        cwd = PyUnicode_AsUTF8String(cwd_from_wchar);
+    #endif
+
+    // decrement the reference count on our temp unicode str to avoid mem leak
+    Py_XDECREF(cwd_from_wchar);
+    returnPyObj = Py_BuildValue("N", cwd);
+
+    CloseHandle(processHandle);
+    free(currentDirectoryContent);
+    return returnPyObj;
+}
+
+
+/*
+Resume or suspends a process.
+*/
+int suspend_resume_process(DWORD pid, int suspend)
+{
+    // a huge thanks to http://www.codeproject.com/KB/threads/pausep.aspx
+    HANDLE         hThreadSnap   = NULL;
+    BOOL           bRet          = FALSE;
+    THREADENTRY32  te32          = {0};
+
+    hThreadSnap = CreateToolhelp32Snapshot(TH32CS_SNAPTHREAD, 0);
+    if (hThreadSnap == INVALID_HANDLE_VALUE)
+        return (FALSE);
+
+    // Fill in the size of the structure before using it.
+    te32.dwSize = sizeof(THREADENTRY32);
+
+    // Walk the thread snapshot to find all threads of the process.
+    // If the thread belongs to the process, add its information
+    // to the display list.
+    if (Thread32First(hThreadSnap, &te32))
+    {
+        do
+        {
+            if (te32.th32OwnerProcessID == pid)
+            {
+				HANDLE hThread = OpenThread(THREAD_SUSPEND_RESUME, FALSE,
+                                            te32.th32ThreadID);
+                if (hThread == NULL) {
+                    CloseHandle(hThread);
+                    PyErr_SetFromWindowsErr(0);
+                    return FALSE;
+                }
+				if (suspend == 1)
+				{
+					//printf("Suspending Thread \n");
+					if (SuspendThread(hThread) == (DWORD)-1) {
+                        CloseHandle(hThread);
+                        PyErr_SetFromWindowsErr(0);
+                        return FALSE;
+                    }
+				}
+				else
+				{
+					//printf("Resuming Thread \n");
+					if (ResumeThread(hThread) == (DWORD)-1) {
+                        CloseHandle(hThread);
+                        PyErr_SetFromWindowsErr(0);
+                        return FALSE;
+                    }
+				}
+				CloseHandle(hThread);
+            }
+        }
+        while (Thread32Next(hThreadSnap, &te32));
+        bRet = TRUE;
+    }
+    else
+        bRet = FALSE;  // could not walk the list of threads
+
+    // Do not forget to clean up the snapshot object.
+    CloseHandle(hThreadSnap);
+
+    return (bRet);
+}
+
+
+static PyObject* suspend_process(PyObject* self, PyObject* args)
+{
+    long pid;
+    int  suspend = 1;
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        PyErr_SetString(PyExc_RuntimeError, "Invalid argument");
+        return NULL;
+    }
+    if (! suspend_resume_process(pid, suspend)){
+        PyErr_SetFromWindowsErr(0);
+        return NULL;
+    }
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+
+static PyObject* resume_process(PyObject* self, PyObject* args)
+{
+    long pid;
+    int suspend = 0;
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        PyErr_SetString(PyExc_RuntimeError, "Invalid argument");
+        return NULL;
+    }
+    if (! suspend_resume_process(pid, suspend)){
+        PyErr_SetFromWindowsErr(0);
+        return NULL;
+    }
+    Py_INCREF(Py_None);
+    return Py_None;
+}
