@@ -1,11 +1,12 @@
 /*
- * $Id: _psutil_mswindows.h 346 2009-04-18 12:40:06Z g.rodola $
+ * $Id: _psutil_mswindows.h 496 2010-02-17 20:59:27Z billiejoex $
  *
  * Windows platform-specific module methods for _psutil_mswindows
  *
  */
 
 #include <Python.h>
+#include <windows.h>
 
 static PyObject* get_pid_list(PyObject* self, PyObject* args);
 static PyObject* kill_process(PyObject* self, PyObject* args);
@@ -21,5 +22,8 @@ static PyObject* get_total_virtmem(PyObject* self, PyObject* args);
 static PyObject* get_avail_phymem(PyObject* self, PyObject* args);
 static PyObject* get_avail_virtmem(PyObject* self, PyObject* args);
 static PyObject* get_system_cpu_times(PyObject* self, PyObject* args);
+static PyObject* get_process_cwd(PyObject* self, PyObject* args);
+static PyObject* suspend_process(PyObject* self, PyObject* args);
+static PyObject* resume_process(PyObject* self, PyObject* args);
 
-
+int suspend_resume_process(DWORD pid, int suspend);
