@@ -1,5 +1,5 @@
 /*
- * $Id: process_info.h 213 2009-03-05 15:48:22Z jloden $
+ * $Id: process_info.h 399 2009-05-12 23:24:21Z jloden $
  *
  * Helper functions related to fetching process information. Used by _psutil_osx
  * module methods.
@@ -10,7 +10,7 @@
 
 typedef struct kinfo_proc kinfo_proc;
 
-int GetBSDProcessList(kinfo_proc **procList, size_t *procCount);
-int getcmdargs(long pid, PyObject **exec_path, PyObject **arglist);
+int get_proc_list(kinfo_proc **procList, size_t *procCount);
+int getcmdargs(long pid, PyObject **exec_path, PyObject **envlist, PyObject **arglist);
 PyObject* get_arg_list(long pid);
 
