@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# $Id: test_psutil.py 374 2009-05-06 05:35:13Z jloden $
+# $Id: test_psutil.py 521 2010-02-27 16:19:10Z billiejoex $
 #
 
 import unittest
@@ -11,7 +11,7 @@ import time
 import signal
 import types
 import errno
-from test import test_support
+import platform
 
 import psutil
 
@@ -28,12 +28,19 @@ def wait_for_pid(pid, timeout=1):
     while 1:
         if pid in psutil.get_pid_list():
 		    # give it one more iteration to allow full initialization
-            time.sleep(0.0001)
+            time.sleep(0.01)
             return
         time.sleep(0.0001)
         if time.time() >= raise_at:
             raise RuntimeError("Timed out")
 
+def kill(pid):
+    """Kill a process given its PID."""
+    if hasattr(os, 'kill'):
+        os.kill(pid, signal.SIGKILL)
+    else:
+        psutil.Process(pid).kill()
+
 
 class TestCase(unittest.TestCase):
 
@@ -42,13 +49,7 @@ class TestCase(unittest.TestCase):
 
     def tearDown(self):
         if self.proc is not None:
-            try:
-                if hasattr(os, 'kill'):
-                    os.kill(self.proc.pid, signal.SIGKILL)
-                else:
-                    psutil.Process(self.proc.pid).kill()
-            finally:
-                self.proc = None
+            kill(self.proc.pid)
 
     def test_get_process_list(self):
         pids = [x.pid for x in psutil.get_process_list()]
@@ -89,17 +90,17 @@ class TestCase(unittest.TestCase):
     def test_total_virtmem(self):
         x = psutil.total_virtmem()
         self.assertTrue(isinstance(x, int) or isinstance(x, long))
-        self.assertTrue(x > 0)
+        self.assertTrue(x >= 0)
 
     def test_used_virtmem(self):
         x = psutil.used_virtmem()
         self.assertTrue(isinstance(x, int) or isinstance(x, long))
-        self.assertTrue(x > -1)
+        self.assertTrue(x >= 0)
 
     def test_avail_virtmem(self):
         x = psutil.avail_virtmem()
         self.assertTrue(isinstance(x, int) or isinstance(x, long))
-        self.assertTrue(x > 0)
+        self.assertTrue(x >= 0)
 
     def test_system_cpu_times(self):
         total = 0
@@ -157,9 +158,10 @@ class TestCase(unittest.TestCase):
 
         # Use time.time() as base value to compare our result using a
         # tolerance of +/- 1 second.
-        # It will fail if the difference between the values is > 1s.
-        if (max([now, create_time]) - min([now, create_time])) > 1:
-            self.fail("expected: %s, found: %s" %(now, create_time))
+        # It will fail if the difference between the values is > 2s.
+        difference = abs(create_time - now)
+        if difference > 2:
+            self.fail("expected: %s, found: %s, difference: %s" %(now, create_time, difference))
 
         # make sure returned value can be pretty printed with strftime
         time.strftime("%Y %m %d %H:%M:%S", time.localtime(p.create_time))
@@ -249,6 +251,44 @@ class TestCase(unittest.TestCase):
             # we expect it to be -1
             self.assertEqual(gid, -1)
 
+    def test_username(self):
+        self.proc = subprocess.Popen(PYTHON, stdout=DEVNULL, stderr=DEVNULL)
+        p = psutil.Process(self.proc.pid)
+        # generic test, only access the attribute (twice for testing the
+        # caching code)
+        p.username
+        p.username
+
+        # UNIX tests
+        if os.name == 'posix':
+            import pwd
+            user = pwd.getpwuid(p.uid).pw_name
+            self.assertEqual(p.username, user)
+
+        # Windows tests
+        elif sys.platform.lower().startswith("win32"):
+            expected_username = os.environ['USERNAME']
+            expected_domain = os.environ['USERDOMAIN']
+            domain, username = p.username.split('\\')
+            self.assertEqual(domain, expected_domain)
+            self.assertEqual(username, expected_username)
+
+    if hasattr(psutil.Process, "getcwd"):
+        def test_getcwd(self):
+            self.proc = subprocess.Popen(PYTHON, stdout=DEVNULL, stderr=DEVNULL)
+            wait_for_pid(self.proc.pid)
+            p = psutil.Process(self.proc.pid)
+            self.assertEqual(p.getcwd(), os.getcwd())
+
+        def test_getcwd_2(self):
+            cmd = [PYTHON, "-c", "import os; os.chdir('..'); input()"]
+            self.proc = subprocess.Popen(cmd, stdout=DEVNULL)
+            wait_for_pid(self.proc.pid)
+            p = psutil.Process(self.proc.pid)
+            time.sleep(0.1)
+            expected_dir = os.path.dirname(os.getcwd())
+            self.assertEqual(p.getcwd(), expected_dir)
+
     def test_parent_ppid(self):
         this_parent = os.getpid()
         self.proc = subprocess.Popen(PYTHON, stdout=DEVNULL, stderr=DEVNULL)
@@ -256,6 +296,12 @@ class TestCase(unittest.TestCase):
         self.assertEqual(p.ppid, this_parent)
         self.assertEqual(p.parent.pid, this_parent)
 
+    def test_suspend_resume(self):
+        self.proc = subprocess.Popen(PYTHON, stdout=DEVNULL, stderr=DEVNULL)
+        p = psutil.Process(self.proc.pid)
+        p.suspend()
+        p.resume()
+
     def test_get_pid_list(self):
         plist = [x.pid for x in psutil.get_process_list()]
         pidlist = psutil.get_pid_list()
@@ -263,6 +309,15 @@ class TestCase(unittest.TestCase):
         # make sure every pid is unique
         self.assertEqual(len(pidlist), len(set(pidlist)))
 
+    def test_test(self):
+        # test for psutil.test() function
+        stdout = sys.stdout
+        sys.stdout = DEVNULL
+        try:
+            psutil.test()
+        finally:
+            sys.stdout = stdout
+
     def test_types(self):
         self.proc = subprocess.Popen(PYTHON, stdout=DEVNULL, stderr=DEVNULL)
         p = psutil.Process(self.proc.pid)
@@ -275,6 +330,10 @@ class TestCase(unittest.TestCase):
         self.assert_(isinstance(p.uid, int))
         self.assert_(isinstance(p.gid, int))
         self.assert_(isinstance(p.create_time, float))
+        self.assert_(isinstance(p.username, str) or \
+                     isinstance(p.username, type(u'')))
+        if hasattr(p, 'getpwd'):
+            self.assert_(isinstance(p.getpwd(), str))
         self.assert_(isinstance(p.is_running(), bool))
         self.assert_(isinstance(p.get_cpu_times(), tuple))
         self.assert_(isinstance(p.get_cpu_times()[0], float))
@@ -320,29 +379,33 @@ class TestCase(unittest.TestCase):
         self.assertRaises(psutil.NoSuchProcess, getattr, p, "cmdline")
         self.assertRaises(psutil.NoSuchProcess, getattr, p, "uid")
         self.assertRaises(psutil.NoSuchProcess, getattr, p, "gid")
+        self.assertRaises(psutil.NoSuchProcess, getattr, p, "create_time")
+        self.assertRaises(psutil.NoSuchProcess, getattr, p, "username")
+        if hasattr(p, 'getcwd'):
+            self.assertRaises(psutil.NoSuchProcess, p.getcwd)
+        self.assertRaises(psutil.NoSuchProcess, p.suspend)
+        self.assertRaises(psutil.NoSuchProcess, p.resume)
         self.assertRaises(psutil.NoSuchProcess, p.kill)
-        # XXX - these tests are supposed to work everywhere except
-        # Windows which keeps returning info for a dead process.
-        if not sys.platform.lower().startswith("win32"):
-            self.assertRaises(psutil.NoSuchProcess, p.get_cpu_times)
-            self.assertRaises(psutil.NoSuchProcess, p.get_cpu_percent)
-            self.assertRaises(psutil.NoSuchProcess, p.get_memory_info)
-            self.assertRaises(psutil.NoSuchProcess, p.get_memory_percent)
+        self.assertRaises(psutil.NoSuchProcess, p.get_cpu_times)
+        self.assertRaises(psutil.NoSuchProcess, p.get_cpu_percent)
+        self.assertRaises(psutil.NoSuchProcess, p.get_memory_info)
+        self.assertRaises(psutil.NoSuchProcess, p.get_memory_percent)
         self.assertFalse(p.is_running())
 
     def test_fetch_all(self):
         valid_procs = 0
+        attrs = ['__str__', 'create_time', 'username', 'getcwd', 'get_cpu_times',
+                 'get_cpu_percent', 'get_memory_info', 'get_memory_percent']
         for p in psutil.process_iter():
-            try:
-                str(p)
-                p.create_time
-                p.get_cpu_times()
-                p.get_cpu_percent()
-                p.get_memory_info()
-                p.get_memory_percent()
-                valid_procs += 1
-            except (psutil.NoSuchProcess, psutil.AccessDenied):
-                continue
+            for attr in attrs:
+                try:
+                    attr = getattr(p, attr, None)
+                    if attr is not None and callable(attr):
+                        attr()
+                    valid_procs += 1
+
+                except (psutil.NoSuchProcess, psutil.AccessDenied):
+                    continue
 
         # we should always have a non-empty list, not including PID 0 etc.
         # special cases.
@@ -370,6 +433,17 @@ class TestCase(unittest.TestCase):
         else:
             p.get_memory_info()
 
+        # username property
+        if sys.platform.lower().startswith("linux"):
+            self.assertEqual(p.username, 'root')
+        elif sys.platform.lower().startswith('freebsd') or \
+        sys.platform.lower().startswith('darwin'):
+            self.assertEqual(p.username, 'root')
+        elif sys.platform.lower().startswith("win32"):
+            self.assertEqual(p.username, 'NT AUTHORITY\\SYSTEM')
+        else:
+            p.username
+
         # PID 0 is supposed to be available on all platforms
         self.assertTrue(0 in psutil.get_pid_list())
         self.assertTrue(psutil.pid_exists(0))
@@ -379,32 +453,12 @@ class TestCase(unittest.TestCase):
 
     # UNIX specific tests
 
-    if not sys.platform.lower().startswith("win32"):
-
+    if os.name == 'posix':
         if hasattr(os, 'getuid') and os.getuid() > 0:
             def test_unix_access_denied(self):
                 p = psutil.Process(1)
                 self.assertRaises(psutil.AccessDenied, p.kill)
 
-    # Windows specific tests
-
-    if sys.platform.lower().startswith("win32"):
-
-        def test_windows_issue_24(self):
-            p = psutil.Process(0)
-            self.assertRaises(psutil.AccessDenied, p.kill)
-
-        def test_windows_pid_4(self):
-            p = psutil.Process(4)
-            self.assertEqual(p.name, 'System')
-            # use __str__ to access all common Process properties to check
-            # that nothing strange happens
-            str(p)
-            self.assertTrue(p.create_time >= 0.0)
-            rss, vms = p.get_memory_info()
-            self.assertTrue(rss > 0)
-            self.assertEqual(vms, 0)
-
 
 if hasattr(os, 'getuid'):
     class LimitedUserTestCase(TestCase):
@@ -426,15 +480,59 @@ if hasattr(os, 'getuid'):
             os.seteuid(self.PROCESS_GID)
             TestCase.tearDown(self)
 
+        if sys.platform.lower().startswith("linux"):
+
+            # disable these tests on Linux as they always raises AccessDenied
+            def test_getcwd(self):
+                return
+            def test_getcwd_2(self):
+                return
+
 
 def test_main():
+    tests = []
     test_suite = unittest.TestSuite()
-    test_suite.addTest(unittest.makeSuite(TestCase))
+
+    tests.append(TestCase)
+
     if hasattr(os, 'getuid') and os.getuid() == 0:
-        test_suite.addTest(unittest.makeSuite(LimitedUserTestCase))
+        tests.append(LimitedUserTestCase)
+
+    if os.name == 'posix':
+        from _posix import PosixSpecificTestCase
+        tests.append(PosixSpecificTestCase)
+
+    # import the specific platform test suite
+    if sys.platform.lower().startswith("linux"):
+        from _linux import LinuxSpecificTestCase as stc
+    elif sys.platform.lower().startswith("win32"):
+        from _windows import WindowsSpecificTestCase as stc
+    elif sys.platform.lower().startswith("darwin"):
+        from _osx import OSXSpecificTestCase as stc
+    elif sys.platform.lower().startswith("freebsd"):
+        from _bsd import BSDSpecificTestCase as stc
+    tests.append(stc)
+
+    for test_class in tests:
+        test_suite.addTest(unittest.makeSuite(test_class))
+
     unittest.TextTestRunner(verbosity=2).run(test_suite)
-    if hasattr(test_support, "reap_children"):  # python 2.5 and >
-        test_support.reap_children()
+
+    # reap_children() function should help ensure that no extra children
+    # (zombies) stick around to hog resources and create problems when
+    # looking for refleaks.
+    # We check for ImportError as on most Unixen python tests are not
+    # installed by default (e.g. on Ubuntu they get installed with:
+    # "apt-get install python-dev")
+    try:
+        from test import test_support
+    except ImportError:
+        pass
+    else:
+        # available since python 2.5
+        if hasattr(test_support, "reap_children"):
+            test_support.reap_children()
+
     DEVNULL.close()
 
 if __name__ == '__main__':
